@@ -3,8 +3,9 @@ class DatasetTypes:
     IC_MULTICLASS = 'classification_multiclass'
     OD = 'object_detection'
     MULTITASK = 'multitask'
+    IMCAP = 'image_caption'
 
-    VALID_TYPES = [IC_MULTILABEL, IC_MULTICLASS, OD, MULTITASK]
+    VALID_TYPES = [IC_MULTILABEL, IC_MULTICLASS, OD, MULTITASK, IMCAP]
 
     @staticmethod
     def is_classification(dataset_type):
